'use strict';

var fs = require('fs');
var path = require('path');
var chalk = require('chalk');
var yargs = require('yargs');
var remark = require('remark');
var toc = require('remark-toc');
var apiDocs = require('@structured-types/api-docs');
var reactPlugin = require('@structured-types/react-plugin');
var propTypesPlugin = require('@structured-types/prop-types-plugin');
var api = require('@structured-types/api');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n["default"] = e;
  return Object.freeze(n);
}

var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var chalk__default = /*#__PURE__*/_interopDefaultLegacy(chalk);
var yargs__namespace = /*#__PURE__*/_interopNamespace(yargs);
var remark__default = /*#__PURE__*/_interopDefaultLegacy(remark);
var toc__default = /*#__PURE__*/_interopDefaultLegacy(toc);
var reactPlugin__default = /*#__PURE__*/_interopDefaultLegacy(reactPlugin);
var propTypesPlugin__default = /*#__PURE__*/_interopDefaultLegacy(propTypesPlugin);

const trimQuotes = (txt) => txt ? txt.replace(/['"]+/g, '') : txt;
const extractCustomTag = (node, tagName) => {
    const nodes = node.children &&
        node.children.filter((child) => child.type === 'html' &&
            child.value &&
            child.value.startsWith(`<${tagName}`));
    return nodes
        ? nodes
            .map((section) => ({
            section,
            match: section.value
                ? section.value.match(/(\w+)\s*=\s*((["'])(.*?)\3|([^>\s]*)(?=\s|\/>))(?=[^<]*>)/g)
                : undefined,
        }))
            .map(({ section, match, }) => ({
            attrs: { section, tagName, node },
            attributes: match
                ? match.map((m) => m.split('=').map((value) => trimQuotes(value)))
                : undefined,
        }))
        : undefined;
};
const inlineNewContent = ({ section, tagName, node }, newContent) => {
    const startTag = `<!-- START-${tagName.toUpperCase()} -->`;
    const endTag = `<!-- END-${tagName.toUpperCase()} -->`;
    const index = node.children ? node.children.indexOf(section) : -1;
    if (index === -1) {
        return;
    }
    let deleteNodes = 0;
    let foundComment = false;
    if (node.children) {
        for (let i = index + 1; i < node.children.length; i += 1) {
            const childNode = node.children[i];
            if (childNode.type == 'html' && childNode.value == startTag) {
                foundComment = true;
            }
            if (foundComment) {
                deleteNodes += 1;
            }
            if (childNode &&
                childNode.type == 'html' &&
                (childNode.value == endTag ||
                    (childNode.value &&
                        childNode.value.toLowerCase().startsWith(`<${tagName}`)))) {
                break;
            }
        }
        node.children.splice(index + 1, deleteNodes, ...[
            {
                type: 'html',
                value: startTag,
            },
            ...newContent,
            {
                type: 'html',
                value: endTag,
            },
        ]);
    }
};

const insertOverview = () => {
    return (node) => {
        const sections = extractCustomTag(node, 'package-section');
        if (sections) {
            sections.forEach(({ attrs, attributes }) => {
                if (attributes) {
                    const file = attributes.find((attribute) => attribute[0] === 'file');
                    const sectionAttr = attributes.find((attribute) => attribute[0] === 'section');
                    const sectionName = sectionAttr
                        ? sectionAttr[1].toLowerCase()
                        : 'overview';
                    if (file) {
                        const content = fs__default["default"].readFileSync(file[1], 'utf8');
                        const overviewNodes = [];
                        const packageJSONFilename = path__default["default"].resolve(path__default["default"].join(path__default["default"].dirname(file[1]), 'package.json'));
                        if (fs__default["default"].existsSync(packageJSONFilename)) {
                            const packageJSON = JSON.parse(fs__default["default"].readFileSync(packageJSONFilename, 'utf8'));
                            const { name, description, repository: { directory }, } = packageJSON;
                            const rootRepoDir = `https://github.com/ccontrols/component-controls/blob/master/${directory}`;
                            overviewNodes.push({
                                type: 'heading',
                                depth: 2,
                                children: [
                                    {
                                        type: 'link',
                                        url: rootRepoDir,
                                        children: [{ type: 'text', value: name }],
                                    },
                                ],
                            });
                            overviewNodes.push({
                                type: 'paragraph',
                                children: [{ type: 'text', value: description }],
                            });
                        }
                        remark__default["default"]()
                            .use(() => (sectionNode) => {
                            let foundDepth = 0;
                            if (sectionNode.children) {
                                for (const section of sectionNode.children) {
                                    if (section.type === 'heading') {
                                        if (section.children &&
                                            section.children.length > 0 &&
                                            section.children[0].type === 'text' &&
                                            section.children[0].value &&
                                            section.children[0].value.toLowerCase() === sectionName) {
                                            foundDepth = section.depth;
                                        }
                                        else if (foundDepth >= section.depth) {
                                            break;
                                        }
                                        else if (foundDepth > 0) {
                                            overviewNodes.push(section);
                                        }
                                    }
                                    else if (foundDepth > 0) {
                                        overviewNodes.push(section);
                                    }
                                }
                            }
                        })
                            .process(content);
                        inlineNewContent(attrs, overviewNodes);
                    }
                }
            });
        }
    };
};

const renderNode = ({ node, ...props }) => {
    if (apiDocs.isHeadingNode(node)) {
        return nodeContent({ node, as: 'heading', depth: node.depth, ...props });
    }
    else if (apiDocs.isParagraphNode(node)) {
        return nodeContent({ node, as: 'paragraph', ...props });
    }
    else if (apiDocs.isBoldNode(node)) {
        return nodeContent({ node, as: 'strong', ...props });
    }
    else if (apiDocs.isEmphasisNode(node)) {
        return nodeContent({ node, as: 'emphasis', ...props });
    }
    else if (apiDocs.isLinkNode(node)) {
        return node.url
            ? nodeContent({ node, as: 'link', url: node.url, ...props })
            : nodeContent({ node, as: 'paragraph', ...props });
    }
    else if (apiDocs.isInlineCodeNode(node)) {
        return nodeContent({ node, as: 'inlineCode', ...props });
    }
    else if (apiDocs.isCodeNode(node)) {
        return nodeContent({ node, as: 'code', ...props });
    }
    else if (apiDocs.isTextNode(node)) {
        return nodeContent({ node, as: 'text', ...props });
    }
    else if (apiDocs.isTableCellNode(node)) {
        return nodeContent({ node, as: 'tableCell', ...props });
    }
    else if (apiDocs.isTableRowNode(node)) {
        return nodeContent({ node, as: 'tableRow', ...props });
    }
    else if (apiDocs.isTableNode(node)) {
        return nodeContent({ node, as: 'table', ...props });
    }
    return null;
};
const nodeContent = ({ node, as, ...rest }) => {
    const value = apiDocs.isNodeWithChildren(node)
        ? nodeComponents({ nodes: node.children, as, ...rest })
        : apiDocs.isNodeWithValue(node)
            ? { type: as, ...rest, value: node.value }
            : null;
    return value;
};
const nodeComponents = ({ nodes, as, ...rest }) => {
    return nodes
        ? {
            type: as,
            children: nodes
                .map((node) => renderNode({ node }))
                .filter((n) => n),
            ...rest,
        }
        : null;
};
const nodesToRemark = (nodes) => {
    return nodes
        .map((node) => {
        const rendered = renderNode({ node });
        return rendered;
    })
        .filter((n) => n);
};

const extractProps = (files, config) => {
    const props = api.parseFiles(files, {
        collectSourceInfo: true,
        collectHelpers: true,
        plugins: [propTypesPlugin__default["default"], reactPlugin__default["default"]],
        ...config,
    });
    const nodes = apiDocs.propsToDocumentation(props, config);
    return nodesToRemark(nodes);
};

/* eslint-disable prefer-spread */
const insertAPISection = (fileName, configFileName) => () => {
    const resolve = (file, filePath) => {
        if (file.startsWith('.')) {
            return path__default["default"].resolve(path__default["default"].dirname(path__default["default"].resolve(filePath)), file);
        }
        return require.resolve(file);
    };
    const splitCommaAttribute = (attributes, name) => {
        if (attributes) {
            const attribute = attributes.find((attribute) => attribute[0] === name);
            return attribute
                ? attribute[1]
                    .split(',')
                    .map((f) => f.trim())
                    .filter((f) => f)
                : undefined;
        }
        return undefined;
    };
    return (node) => {
        const sections = extractCustomTag(node, 'api-readme');
        if (sections) {
            sections.forEach(({ attrs, attributes = [] }) => {
                var _a;
                if (attributes) {
                    const newNodes = [];
                    let elementId = undefined;
                    const inlineOptions = attributes.reduce((acc, attr) => {
                        let value;
                        const strValue = attr[1];
                        if (strValue === 'true') {
                            value = true;
                        }
                        else if (strValue === 'false') {
                            value = false;
                        }
                        else if (!isNaN(parseFloat(strValue))) {
                            value = parseFloat(strValue);
                        }
                        else if (typeof attr[1] === 'string') {
                            if (attr[0] === 'id') {
                                elementId = attr[1];
                                return acc;
                            }
                            value = splitCommaAttribute(attributes, attr[0]);
                        }
                        return { ...acc, [attr[0]]: value };
                    }, { files: [] });
                    const { config = {}, filepath: configFilePath } = apiDocs.apiDocsConfig(fileName, configFileName, elementId) || {};
                    const mergedConfig = apiDocs.mergeConfig(config, inlineOptions);
                    const files = configFilePath && config.files
                        ? config.files.map((f) => resolve(f, configFilePath))
                        : (_a = inlineOptions.files) === null || _a === void 0 ? void 0 : _a.map((f) => resolve(f, fileName));
                    if (!files.length) {
                        const srcFolder = resolve('./src', fileName);
                        if (fs__default["default"].existsSync(srcFolder)) {
                            const sourceFiles = fs__default["default"].readdirSync(srcFolder);
                            const indexFile = sourceFiles.find((f) => f.startsWith('index'));
                            if (indexFile) {
                                files.push(resolve(path__default["default"].resolve(srcFolder, indexFile), fileName));
                            }
                        }
                    }
                    const tsNodes = extractProps(files, mergedConfig);
                    newNodes.push(...tsNodes);
                    inlineNewContent(attrs, newNodes);
                }
            });
        }
    };
};

/**
 * create markdown pages from README.md type files
 * command-line arguments
 * @param {string} f - input file name to be processed, default is README.md
 * @param {boolean} t - whether to generate a table of contents, default is true
 * @param {boolean} l - whether to output logs, default is true
 */
var index = () => {
    const args = yargs__namespace
        .help()
        .alias('help', 'h')
        .option('f', {
        alias: 'file',
        describe: 'Input file name',
        type: 'string',
        default: 'README.md',
    })
        .option('c', {
        alias: 'config',
        describe: 'Configuration file full path',
        type: 'string',
    })
        .option('t', {
        alias: 'toc',
        describe: 'Generate a table of content',
        type: 'boolean',
        default: true,
    })
        .option('l', {
        alias: 'log',
        describe: 'Enabled logs',
        type: 'boolean',
        default: true,
    });
    const options = args.parse(process.argv);
    const fileName = options.f;
    const configFileName = options.c;
    if (options.log) {
        console.log('processing file:', chalk__default["default"].red(path__default["default"].resolve(fileName)));
    }
    let r = remark__default["default"]()
        .use(insertAPISection(fileName, configFileName))
        .use(insertOverview);
    if (options.toc) {
        r = r.use(toc__default["default"], { tight: true });
    }
    r.process(fs__default["default"].readFileSync(fileName, 'utf8'), function (err, file) {
        if (err) {
            throw err;
        }
        fs__default["default"].writeFileSync(fileName, String(file));
    });
};

module.exports = index;
